<?php
declare(strict_types=1);

namespace Catequil\Db\Container;

use Doctrine\DBAL\Connection;
use Doctrine\DBAL\DriverManager;
use Doctrine\DBAL\Exception;
use Laminas\ServiceManager\Exception\ServiceNotCreatedException;
use Laminas\ServiceManager\Factory\AbstractFactoryInterface;
use Psr\Container\ContainerInterface;

/**
 * Class ConnectionAbstractFactory
 *
 * @package Catequil\Db\Container
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-db
 */
final class ConnectionAbstractFactory implements AbstractFactoryInterface
{
    /**
     * Can we create a service with the given name?
     *
     * @param ContainerInterface $container
     * @param $requestedName
     * @return bool
     */
    public function canCreate(ContainerInterface $container, $requestedName): bool
    {
        return is_array($this->getDbConfig($container, $requestedName));
    }

    /**
     * Create a service with the given name.
     *
     * @param ContainerInterface $container
     * @param $requestedName
     * @param array|null $options
     * @return Connection
     */
    public function __invoke(ContainerInterface $container, $requestedName, ?array $options = null): Connection
    {
        $config = $this->getDbConfig($container, $requestedName);

        try {
            return DriverManager::getConnection($config);
        } catch (Exception $e) {
            throw new ServiceNotCreatedException('Could not create database connection.', $e->getCode(), $e);
        }
    }

    private function getDbConfig(ContainerInterface $container, string $requestedName): ?array
    {
        /** @noinspection PhpUnhandledExceptionInspection */
        $config = $container->get('config')['db'] ?? [];

        if ($requestedName === Connection::class) {
            $requestedName = $config['default_connection'] ?? 'db.default';
        }

        return $config['connections'][$requestedName] ?? null;
    }
}