<?php
declare(strict_types=1);

namespace CatequilTest\Db\Container;

use CatequilTest\Db\AbstractTestCase;
use Doctrine\DBAL\Connection;
use Laminas\ServiceManager\Exception\ServiceNotCreatedException;

/**
 * Class ConnectionAbstractFactoryTest
 *
 * @package CatequilTest\Db\Container
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/catequil-db
 */
final class ConnectionAbstractFactoryTest extends AbstractTestCase
{
    protected static array $config = [
        'db' => [
            'connections' => [
                'db.default' => [
                    'driver' => 'pdo_mysql',
                ],
                'db.memory' => [
                    'driver' => 'pdo_sqlite',
                    'memory' => true,
                ],
                'db.invalid' => [
                    'driver' => 'invalid',
                ],
            ],
        ],
    ];

    public function testCanGetDefaultConnection(): void
    {
        $container = self::getContainer();
        /** @noinspection PhpUnhandledExceptionInspection */
        $connection = $container->get(Connection::class);

        self::assertInstanceOf(Connection::class, $connection);
        /** @noinspection PhpUnhandledExceptionInspection */
        self::assertSame($connection, $container->get('db'));
    }

    public function testCanGetConnectionByName(): void
    {
        $container = self::getContainer();
        /** @noinspection PhpUnhandledExceptionInspection */
        $connection = $container->get('db.memory');

        self::assertInstanceOf(Connection::class, $connection);
    }

    public function testWillThrowExceptionForInvalidDriver(): void
    {
        self::expectException(ServiceNotCreatedException::class);

        $container = self::getContainer();
        /** @noinspection PhpUnhandledExceptionInspection */
        $container->get('db.invalid');
    }
}